@@ -233,7 +233,9 @@ public interface A3CanvasInterface {
         EDIT,
         /** EXAMINEモードと似ていますが、注視点が任意に変更できる観覧モードです。 */
         SIMPLE,
-        /** アバタの後方からアバタを追跡するような観覧モードです。 */
+        /** アバタの後方からアバタを追跡するような観覧モードです。A23.setDefaultUpperDirection()などを
+         * 用いてZ軸が上になる座標系に設定している場合には正常に機能しないので、ChaseControllerを
+         * 用いる方法を検討して下さい。 */
         CHASE,
         /** ユーザ定義のナビゲーションモードです。 */
         USER
@@ -593,4 +595,28 @@ public interface A3CanvasInterface {
      * 全てのA3Objectを削除して表示されないようにします。
      */
     public void delAllLockedA3(int scene);
+    /**
+     * 上方向をY軸とするのかZ軸とするのかの変更を行う。
+     * デフォルトはY軸で、この場合は特に何もかわらないが、
+     * Z軸が設定された場合は表示されるA3Objectが
+     * 自動的に回転されて正常な向きで表示されるようになる。
+     */
+    public void setUpperDirection(A3Object.UpperDirection d);
+    /**
+     * シーンを指定して、上方向をY軸とするのかZ軸とするのかの
+     * 変更を行う。
+     * デフォルトはY軸で、この場合は特に何もかわらないが、
+     * Z軸が設定された場合は表示されるA3Objectが
+     * 自動的に回転されて正常な向きで表示されるようになる。
+     */
+    public void setUpperDirection(A3Object.UpperDirection d,int scene);
+    /**
+     * 上方向をY軸としているのかZ軸としているのかを得るための関数です。
+     */
+    public A3Object.UpperDirection getUpperDirection();
+    /**
+     * シーンを指定して、上方向をY軸としているのかZ軸としているのか
+     * を得るための関数です。
+     */
+    public A3Object.UpperDirection getUpperDirection(int scene);
 }
