@@ -845,4 +845,36 @@ public class A3Screen implements A3CanvasInterface {
     public void delAllLockedA3(int scene) {
         canvas.delAllLockedA3(scene);
     }
+    /**
+     * 上方向をY軸とするのかZ軸とするのかの変更を行う。
+     * デフォルトはY軸で、この場合は特に何もかわらないが、
+     * Z軸が設定された場合は表示されるA3Objectが
+     * 自動的に回転されて正常な向きで表示されるようになる。
+     */
+    public void setUpperDirection(A3Object.UpperDirection d) {
+        canvas.setUpperDirection(d);
+    }
+    /**
+     * シーンを指定して、上方向をY軸とするのかZ軸とするのかの
+     * 変更を行う。
+     * デフォルトはY軸で、この場合は特に何もかわらないが、
+     * Z軸が設定された場合は表示されるA3Objectが
+     * 自動的に回転されて正常な向きで表示されるようになる。
+     */
+    public void setUpperDirection(A3Object.UpperDirection d,int scene) {
+        canvas.setUpperDirection(d,scene);
+    }
+    /**
+     * 上方向をY軸としているのかZ軸としているのかを得るための関数です。
+     */
+    public A3Object.UpperDirection getUpperDirection() {
+        return canvas.getUpperDirection();
+    }
+    /**
+     * シーンを指定して、上方向をY軸としているのかZ軸としているのか
+     * を得るための関数です。
+     */
+    public A3Object.UpperDirection getUpperDirection(int scene) {
+        return canvas.getUpperDirection(scene);
+    }
 }
