@@ -37,6 +37,8 @@ class A3VirtualUniverse extends VirtualUniverse {
             init((Canvas3D)c);
         else if (canvas instanceof JA3Canvas)
             init(((JA3Canvas)canvas).getOffscreenCanvas3D());
+        else if (canvas instanceof A3Widget)
+            init(((A3Widget)canvas).tCanvas.getOffscreenCanvas3D());
     }
     void init(Canvas3D c3d) {
         canvas3d = c3d;
@@ -1040,4 +1042,44 @@ class A3VirtualUniverse extends VirtualUniverse {
         else
             throw new IllegalArgumentException();
     }
+    /**
+     * 上方向をY軸とするのかZ軸とするのかの変更を行う。
+     * デフォルトはY軸で、この場合は特に何もかわらないが、
+     * Z軸が設定された場合は表示されるA3Objectが
+     * 自動的に回転されて正常な向きで表示されるようになる。
+     */
+    public void setUpperDirection(A3Object.UpperDirection d) {
+        scene.setUpperDirection(d);
+    }
+    /**
+     * シーンを指定して、上方向をY軸とするのかZ軸とするのかの
+     * 変更を行う。
+     * デフォルトはY軸で、この場合は特に何もかわらないが、
+     * Z軸が設定された場合は表示されるA3Objectが
+     * 自動的に回転されて正常な向きで表示されるようになる。
+     */
+    public void setUpperDirection(A3Object.UpperDirection d,int scene) {
+        Scene s = scenes.get(scene);
+        if (s!=null)
+            s.setUpperDirection(d);
+        else
+            throw new IllegalArgumentException();
+    }
+    /**
+     * 上方向をY軸としているのかZ軸としているのかを得るための関数です。
+     */
+    public A3Object.UpperDirection getUpperDirection() {
+        return scene.upperDirection;
+    }
+    /**
+     * シーンを指定して、上方向をY軸としているのかZ軸としているのか
+     * を得るための関数です。
+     */
+    public A3Object.UpperDirection getUpperDirection(int scene) {
+        Scene s = scenes.get(scene);
+        if (s!=null)
+            return s.upperDirection;
+        else
+            throw new IllegalArgumentException();
+    }
 }
