@@ -5,16 +5,20 @@ import java.net.*;
 import java.util.*;
 
 import javax.media.j3d.*;
+
 import org.jdesktop.j3d.loaders.vrml97.VrmlLoader;
 import javax.vecmath.*;
 
 
 import org.w3c.dom.Attr;
+import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 import org.w3c.dom.NamedNodeMap;
 //import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 import org.w3c.dom.Text;
+import org.w3c.dom.bootstrap.DOMImplementationRegistry;
+import org.w3c.dom.ls.*;
 
 import com.sun.j3d.loaders.Scene;
 
@@ -62,6 +66,11 @@ class Action3DData {
     double minWalkSpeed = 0.1;
     double minRunSpeed = 1.0;
     boolean autoActionControl = false;
+    String rdf;
+    String tags[];
+    String profiles[];
+    String thumbnails[];
+    String html;
 
     Action actions[];
     String actionNames[];
@@ -78,6 +87,8 @@ class Action3DData {
         double scale = 1.0;
         double[] offsetXYZ;
         double[] rotationXYZ;
+        TransformGroup tg;
+        Node shape;
     }
     //アクションデータを保存しておくためのクラス
     class Action {
@@ -90,11 +101,21 @@ class Action3DData {
         String soundType = "PointSound";
         float soundGain = 1.0f;
         float soundOffsetXYZ[] = {0.0f,0.0f,0.0f};
+        float soundDirectionXYZ[] = {0.0f,0.0f,1.0f};
+        boolean soundContinue = true;
         Sound sound;
         double scale;
         double[] offsetXYZ;
         double[] rotationXYZ;
         HashMap<String,Part> pHash;
+        double segno;
+        double dalsegno;
+        A3Object.BalloonDir balloonDirection = A3Object.BalloonDir.RIGHT;
+        double rightBalloonOffset[] = {0.0,0.0};
+        double leftBalloonOffset[] = {0.0,0.0};
+        double topBalloonOffset[] = {0.0,0.0};
+        double bottomBalloonOffset[] = {0.0,0.0};
+        double labelOffset[] = {0.0,0.0};
 
         Group rootGroup;
         ActionBehavior behavior;
@@ -112,11 +133,20 @@ class Action3DData {
             ret.soundType = soundType;
             ret.soundGain = soundGain;
             ret.soundOffsetXYZ = soundOffsetXYZ.clone();
+            ret.soundDirectionXYZ = soundDirectionXYZ.clone();
+            ret.soundContinue = soundContinue;
             ret.scale = scale;
             ret.rotationXYZ = (rotationXYZ==null)?null:rotationXYZ.clone();
             ret.offsetXYZ = (offsetXYZ==null)?null:offsetXYZ.clone();
             ret.pHash = new HashMap<String,Part>(pHash);
             ret.motion = motion;
+            ret.segno = segno;
+            ret.dalsegno = dalsegno;
+            ret.rightBalloonOffset = rightBalloonOffset.clone();
+            ret.leftBalloonOffset = leftBalloonOffset.clone();
+            ret.topBalloonOffset = topBalloonOffset.clone();
+            ret.bottomBalloonOffset = bottomBalloonOffset.clone();
+            ret.labelOffset = labelOffset.clone();
 
             return ret;
         }
@@ -154,7 +184,10 @@ class Action3DData {
                 rootGroup.addChild(tg0);
             else
                 rootGroup.addChild(tg);
-            motion = motions.get(bvhFile);
+            if (bvhFile.equals("none"))
+                motion = new DummyMotion();
+            else
+                motion = motions.get(bvhFile);
             String rootBoneName = motion.getRootBone();
             Part p = pHash.get(rootBoneName);
             if (p!=null) {
@@ -187,6 +220,8 @@ class Action3DData {
                     n = tg9;
                 }
                 tg.addChild(n);
+                p.tg = tg;
+                p.shape = n;
 
                 if (sabf.b!=null)
                     for (int i=0;i<sabf.b.length;i++) 
@@ -196,17 +231,69 @@ class Action3DData {
                         rootGroup.addChild(sabf.f[i].cloneNode(false));
             }
             tgMap.put(rootBoneName,tg);
-            construct(tg,rootBoneName,tgMap);
+            construct(tg,rootBoneName);
 
             if (soundFile!=null) {
                 MediaContainer mc = sounds.get(soundFile);
                 if (soundType.equals("BackgroundSound")) {
                     sound = new BackgroundSound(mc,soundGain);
-                } else {
+                } else if (soundType.equals("PointSound")){
                     sound = new PointSound(mc,soundGain,
                                            soundOffsetXYZ[0],
                                            soundOffsetXYZ[1],
                                            soundOffsetXYZ[2]);
+                } else if (soundType.equals("ConeSound")) {
+                    /*
+                    sound = new ConeSound(mc,soundGain,
+                            soundOffsetXYZ[0],
+                            soundOffsetXYZ[1],
+                            soundOffsetXYZ[2],
+                            soundDirectionXYZ[0],
+                            soundDirectionXYZ[1],
+                            soundDirectionXYZ[2]);
+                    */
+
+                    //http://old.siggraph.org/education/materials/siggraph_courses/S99/40/java3d4p.pdf
+                    //さっぱり意味がわかりません。
+                    Point2f[] myFrontAtten = {
+                            new Point2f( 100.0f, 1.0f ),
+                            new Point2f( 350.0f, 0.5f ),
+                            new Point2f( 600.0f, 0.0f )
+                            };
+                            Point2f[] myBackAtten = {
+                            new Point2f( 50.0f, 1.0f ),
+                            new Point2f( 100.0f, 0.5f ),
+                            new Point2f( 200.0f, 0.0f )
+                            };
+                            Point3f[] myAngular = {
+                            new Point3f( 0.000f, 1.0f, 20000.0f ),
+                            new Point3f( 0.785f, 0.5f, 5000.0f ),
+                            new Point3f( 1.571f, 0.0f, 2000.0f ),
+                            };
+                    sound = new ConeSound();
+                    ((ConeSound)sound).setSoundData(mc);
+                    ((ConeSound)sound).setInitialGain(soundGain);
+                    ((ConeSound)sound).setPosition(soundOffsetXYZ[0],soundOffsetXYZ[1],soundOffsetXYZ[2]);
+                    ((ConeSound)sound).setDirection(soundDirectionXYZ[0],soundDirectionXYZ[1],soundDirectionXYZ[2]);
+                    ((ConeSound)sound).setDistanceGain(myFrontAtten,myBackAtten);
+                    ((ConeSound)sound).setAngularAttenuation(myAngular);
+
+                    /*
+                    //http://www.koders.com/java/fid9234663F18A13A2F1B99E463FCFE7B1D75BCB621.aspx?s=DistanceAttenuation#L40
+                    //さっぱり意味がわかりません。
+                    sound = new ConeSound(mc,soundGain,
+                            soundOffsetXYZ[0],
+                            soundOffsetXYZ[1],
+                            soundOffsetXYZ[2],
+                            soundDirectionXYZ[0],
+                            soundDirectionXYZ[1],
+                            soundDirectionXYZ[2]);
+                    float angle = (float) (Math.PI / 4);
+                    ((ConeSound)sound).setAngularAttenuation(new Point2f[] {
+                        new Point2f(angle, 0)});
+                    float distanceAtZero = 40;
+                    ((ConeSound)sound).setDistanceGain(new float []{0, distanceAtZero}, new float []{1, 0});
+                    */
                 }
                 sound.setEnable(false);
                 sound.setPause(false);
@@ -223,19 +310,24 @@ class Action3DData {
                 Link ll = new Link(sg);
                 rootGroup.addChild(ll);
             }
-        }
-
-        void construct(Group parent,String parentName,HashMap<String,TransformGroup> tgMap) {
-            String children[] = motion.getChildBones(parentName);
-            for (int i=0;i<children.length;i++) {
-                TransformGroup tg = new TransformGroup();
-                tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
-                parent.addChild(tg);
-                Part p = pHash.get(children[i]);
-                if (p!=null) {
+            //boneに関連付けされていないPartの処理
+            String bones[] = motion.getAllBones();
+            String parts[] = pHash.keySet().toArray(new String[0]);
+            for (int i=0;i<parts.length;i++) {
+                boolean exist = false;
+                for (int j=0;j<bones.length;j++) {
+                    if (parts[i].equals(bones[j])) {
+                        exist = true;
+                        break;
+                    }
+                }
+                if (!exist) {
+                    //syori
+                    p = pHash.get(parts[i]);
+                    Node n = null;
                     ShapeAndBF sabf = shapes.get(p.fileName);
                     if ((p.rotationXYZ==null)&&(p.offsetXYZ==null)&&(p.scale==1.0)) {
-                        tg.addChild(new Link(sabf.sg));
+                        n = new Link(sabf.sg);
                     } else {
                         Transform3D t9a = new Transform3D();
                         Transform3D t9b = new Transform3D();
@@ -258,8 +350,62 @@ class Action3DData {
                         }
                         TransformGroup tg9 = new TransformGroup(t9a);
                         tg9.addChild(new Link(sabf.sg));
-                        tg.addChild(tg9);
+                        n = tg9;
                     }
+                    tg.addChild(n);
+                    p.tg = tg;
+                    p.shape = n;
+
+                    if (sabf.b!=null)
+                        for (int j=0;j<sabf.b.length;j++) 
+                            rootGroup.addChild(sabf.b[j].cloneNode(false));
+                    if (sabf.f!=null)
+                        for (int j=0;j<sabf.f.length;j++) 
+                            rootGroup.addChild(sabf.f[j].cloneNode(false));
+                }
+            }
+        }
+
+        void construct(Group parent,String parentName) {
+            String children[] = motion.getChildBones(parentName);
+            for (int i=0;i<children.length;i++) {
+                TransformGroup tg = new TransformGroup();
+                tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
+                parent.addChild(tg);
+                Part p = pHash.get(children[i]);
+                if (p!=null) {
+                    Node n = null;
+                    ShapeAndBF sabf = shapes.get(p.fileName);
+                    if ((p.rotationXYZ==null)&&(p.offsetXYZ==null)&&(p.scale==1.0)) {
+                        n = new Link(sabf.sg);
+                    } else {
+                        Transform3D t9a = new Transform3D();
+                        Transform3D t9b = new Transform3D();
+                        t9a.set(Transform3D.IDENTITY);
+                        t9b.set(Transform3D.IDENTITY);
+                        if (p.rotationXYZ!=null) {
+                            t9a.rotX(p.rotationXYZ[0]/180.0*Math.PI);
+                            t9b.rotY(p.rotationXYZ[1]/180.0*Math.PI);
+                            t9a.mul(t9b);
+                            t9b.rotZ(p.rotationXYZ[2]/180.0*Math.PI);
+                            t9a.mul(t9b);
+                        }
+                        t9b.set(Transform3D.IDENTITY);
+                        t9b.setScale(p.scale);
+                        t9a.mul(t9b);
+                        if (p.offsetXYZ!=null) {
+                            t9b.set(Transform3D.IDENTITY);
+                            t9b.setTranslation(new Vector3d(p.offsetXYZ[0],p.offsetXYZ[1],p.offsetXYZ[2]));
+                            t9a.mul(t9b);
+                        }
+                        TransformGroup tg9 = new TransformGroup(t9a);
+                        tg9.addChild(new Link(sabf.sg));
+                        n = tg9;
+                    }
+                    tg.addChild(n);
+                    p.tg = tg;
+                    p.shape = n;
+
                     if (sabf.b!=null)
                         for (int j=0;j<sabf.b.length;j++) 
                             rootGroup.addChild(sabf.b[j].cloneNode(false));
@@ -268,7 +414,7 @@ class Action3DData {
                             rootGroup.addChild(sabf.f[j].cloneNode(false));
                 }
                 tgMap.put(children[i],tg);
-                construct(tg,children[i],tgMap);
+                construct(tg,children[i]);
             }
         }
 
@@ -277,17 +423,17 @@ class Action3DData {
         }
 
         void start() {
-            if (sound!=null) {
-                sound.setEnable(true);
-                //sound.setPause(false);
-            }
+            //if (sound!=null) {
+            //    sound.setEnable(true);
+            //    //sound.setPause(false);
+            //}
         }
 
         void stop() {
-            if (sound!=null) {
-                sound.setEnable(false);
-                //sound.setPause(true);
-            }
+            //if (sound!=null) {
+            //    sound.setEnable(false);
+            //    //sound.setPause(true);
+            //}
         }
 
         boolean isStoped() {
@@ -341,6 +487,11 @@ class Action3DData {
         ret.minWalkSpeed = minWalkSpeed;
         ret.minRunSpeed = minRunSpeed;
         ret.autoActionControl = autoActionControl;
+        ret.rdf = rdf;
+        ret.tags = tags.clone();
+        ret.profiles = profiles.clone();
+        ret.thumbnails = thumbnails.clone();
+        ret.html = html;
 
         ret.actions = new Action[actions.length];
         for (int i=0;i<actions.length;i++) {
@@ -400,7 +551,44 @@ class Action3DData {
         comment = a3.getC();
         actions = new Action[a3.getA().size()];
         actionNames = new String[a3.getA().size()];
+        rdf = domSerialize(a3.getAny());
+        tags = new String[a3.getTag().size()];
         int i=0;
+        for (Tag t:a3.getTag()) {
+            tags[i++] = t.getName();
+        }
+        profiles = new String[a3.getProfile().size()];
+        i=0;
+        for (Profile p:a3.getProfile()) {
+            profiles[i++] = p.getUri();
+        }
+        thumbnails = new String[a3.getThumbnail().size()];
+        i=0;
+        for (Thumbnail t:a3.getThumbnail()) {
+            if (url.getProtocol().equals("file")) {
+                File f = new File(url.toURI());
+                if (f.isDirectory()) {
+                    thumbnails[i++] = url.toString()+"/"+t.getSrc();
+                } else {
+                    thumbnails[i++] = "x-rzip:"+url.toString()+"!/"+t.getSrc();
+                }
+            } else {
+                thumbnails[i++] = "x-rzip:"+url.toString()+"!/"+t.getSrc();
+            }
+        }
+        if (a3.getHtmlfile()!=null) {
+            if (url.getProtocol().equals("file")) {
+                File f = new File(url.toURI());
+                if (f.isDirectory()) {
+                    html = url.toString()+"/"+a3.getHtmlfile().getSrc();
+                } else {
+                    html = "x-rzip:"+url.toString()+"!/"+a3.getHtmlfile().getSrc();
+                }
+            } else {
+                html = "x-rzip:"+url.toString()+"!/"+a3.getHtmlfile().getSrc();
+            }
+        }
+        i=0;
         for (A aNode : a3.getA()) {
             Action a = new Action();
             a.actionNo = i;
@@ -413,6 +601,13 @@ class Action3DData {
             a.rotationXYZ = makeDouble3(aNode.getRot());
             a.offsetXYZ = makeDouble3(aNode.getOffset());
             a.pHash = new HashMap<String,Part>();
+            a.segno = aNode.getSegno();
+            a.dalsegno = aNode.getDalsegno();
+            a.rightBalloonOffset = makeDouble2(aNode.getRightBalloonOffset());
+            a.leftBalloonOffset = makeDouble2(aNode.getLeftBalloonOffset());
+            a.topBalloonOffset = makeDouble2(aNode.getTopBalloonOffset());
+            a.bottomBalloonOffset = makeDouble2(aNode.getBottomBalloonOffset());
+            a.labelOffset = makeDouble2(aNode.getLabelOffset());
             for (P pNode : aNode.getP()) {
                 Part p = new Part();
                 String partName = pNode.getName().trim();
@@ -434,6 +629,8 @@ class Action3DData {
                     a.soundType = "ConeSound";
                 a.soundGain = (float)s.getGain();
                 a.soundOffsetXYZ = makeFloat3(s.getOffset());
+                a.soundDirectionXYZ = makeFloat3(s.getDirection());
+                a.soundContinue = s.isContinue();
             }
             i++;
         }
@@ -450,6 +647,18 @@ class Action3DData {
         }
         return ret;
     }
+    double[] makeDouble2(String s) {
+        //if (s.equals("0.0 0.0"))
+        //    return null;
+        double ret[] = new double[2];
+        StringTokenizer st = new StringTokenizer(s);
+        int i=0;
+        while (st.hasMoreTokens()) {
+            ret[i] = Double.parseDouble(st.nextToken());
+            i++;
+        }
+        return ret;
+    }
     float[] makeFloat3(String s) {
         //if (s.equals("0.0 0.0 0.0"))
         //    return null;
@@ -462,6 +671,23 @@ class Action3DData {
         }
         return ret;
     }
+    String domSerialize(org.w3c.dom.Element e) {
+        try {
+            DOMImplementationRegistry dir = DOMImplementationRegistry.newInstance();
+            DOMImplementation di = dir.getDOMImplementation("LS 3.0");
+            DOMImplementationLS dils = (DOMImplementationLS)di;
+            LSOutput output = dils.createLSOutput();
+            ByteArrayOutputStream baos = new ByteArrayOutputStream();
+            output.setByteStream(baos);
+            output.setEncoding("UTF-8");
+            LSSerializer serializer = dils.createLSSerializer();
+            serializer.write(e, output);
+            return baos.toString("UTF-8");
+        } catch (Exception e1) {
+            e1.printStackTrace();
+            return "";
+        }
+    }
     void loadCatalog_OLD() throws Exception {
         DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
         dbf.setNamespaceAware(true);
@@ -709,6 +935,8 @@ class Action3DData {
                 s.add(actions[i].bvhFile);
         motions = new HashMap<String,Motion>();
         for (String filename : s) {
+            if (filename.equals("none"))
+                continue;
             URL motionURL = null;
             if (url.getProtocol().equals("file")) {
                 File f = new File(url.toURI());
@@ -770,7 +998,7 @@ class Action3DData {
         //allActionSwitch.setWhichChild(0);
     }
 
-//------------------------------------------------------------
+  //------------------------------------------------------------
 
     Node getNode() {
         allActionBranchGroup.detach();
@@ -890,4 +1118,44 @@ class Action3DData {
     double getMinRunSpeed() {
         return minRunSpeed;
     }
+    //------------------------------------------------------------
+    void setShape(int actionNo,String boneName,Node shape) {
+        Part p = actions[actionNo].pHash.get(boneName);
+        p.tg.removeChild(p.shape);
+        p.shape = shape;
+        p.tg.addChild(shape);
+    }
+    void setShape(Serializable action,String boneName,Node shape) {
+        if (action instanceof String) {
+            int i = getActionNoFromActionName((String)action);
+            setShape(i,boneName,shape);
+        } else if (action instanceof Integer) {
+            int i = ((Integer)action).intValue();
+            setShape(i,boneName,shape);
+        }
+    }
+    //------------------------------------------------------------
+    class DummyMotion implements Motion {
+        public double getMotionLength() {
+            return 0.001;
+        }
+        public double getDefaultFrameTime() {
+            return 0.0333;
+        }
+        public String getRootBone() {
+            return null;
+        }
+        public String getParentBone(String b) {
+            return null;
+        }
+        public String[] getChildBones(String b) {
+            return new String[0];
+        }
+        public String[] getAllBones() {
+            return new String[0];
+        }
+        public Transform3D getTransform3D(String bone,double time) {
+            return new Transform3D();
+        }
+    }
 }
