@@ -96,8 +96,6 @@ class FlyController extends A3Controller implements Runnable {
         Quat4d q = new Quat4d(0.0,Math.sin(t),0.0,Math.cos(t));
         cameraQ.mul(q);
         a3canvas.setCameraLocImmediately(cameraV);
-cameraQ.z=0.0;
-cameraQ.normalize();
         a3canvas.setCameraQuatImmediately(cameraQ);
     }
     //NAVIモード(首ふり)
@@ -111,8 +109,6 @@ cameraQ.normalize();
         t = shiftX * -0.001;
         q = new Quat4d(0.0,Math.sin(t),0.0,Math.cos(t));
         cameraQ.mul(q);
-cameraQ.z=0.0;
-cameraQ.normalize();
         a3canvas.setCameraQuatImmediately(cameraQ);
     }
     //NAVIモード(平行移動)
