@@ -6,7 +6,8 @@ import javax.media.j3d.*;
 import javax.vecmath.*;
 
 import java.io.*;
-import java.util.*;
+import java.util.Arrays;
+
 import javax.imageio.ImageIO;
 
 import com.sun.j3d.exp.swing.JCanvas3D;
@@ -496,9 +497,15 @@ public class JA3Canvas extends JCanvas3D implements A3CanvasInterface {
     }
     public void paintComponent(Graphics g) {
         super.paintComponent(g);
-        ArrayList<Component2D> cs = universe.getComponents2D();
+        Component2D cs[] = universe.getComponents2D().toArray(new Component2D[0]);
+        for (Component2D c: cs) {
+            c.calPhysicalZ(this);
+        }
+        Arrays.sort(cs,Component2DComparator.comparator);
         Graphics2D gg = (Graphics2D)g;
         for (Component2D c : cs) {
+            if (c.z>0.0)
+                continue;
             c.paint(gg,this);
         }
         //gg.flush(true);
@@ -857,4 +864,36 @@ public class JA3Canvas extends JCanvas3D implements A3CanvasInterface {
     public void delAllLockedA3(int scene) {
         universe.delAllLockedA3(scene);
     }
+    /**
+     * 上方向をY軸とするのかZ軸とするのかの変更を行う。
+     * デフォルトはY軸で、この場合は特に何もかわらないが、
+     * Z軸が設定された場合は表示されるA3Objectが
+     * 自動的に回転されて正常な向きで表示されるようになる。
+     */
+    public void setUpperDirection(A3Object.UpperDirection d) {
+        universe.setUpperDirection(d);
+    }
+    /**
+     * シーンを指定して、上方向をY軸とするのかZ軸とするのかの
+     * 変更を行う。
+     * デフォルトはY軸で、この場合は特に何もかわらないが、
+     * Z軸が設定された場合は表示されるA3Objectが
+     * 自動的に回転されて正常な向きで表示されるようになる。
+     */
+    public void setUpperDirection(A3Object.UpperDirection d,int scene) {
+        universe.setUpperDirection(d,scene);
+    }
+    /**
+     * 上方向をY軸としているのかZ軸としているのかを得るための関数です。
+     */
+    public A3Object.UpperDirection getUpperDirection() {
+        return universe.getUpperDirection();
+    }
+    /**
+     * シーンを指定して、上方向をY軸としているのかZ軸としているのか
+     * を得るための関数です。
+     */
+    public A3Object.UpperDirection getUpperDirection(int scene) {
+        return universe.getUpperDirection(scene);
+    }
 }
