@@ -92,7 +92,11 @@ class WalkController extends A3Controller implements Runnable {
         Matrix3d m = new Matrix3d();
         m.set(cameraQ);
         m.transform(v);
-        v.y = 0.0;
+        if (a3canvas.getUpperDirection()==A3Object.UpperDirection.Z) {
+            v.z = 0.0;
+        } else {
+            v.y = 0.0;
+        }
         if (v.length()>0.0001)
             v.normalize();
         double s = 0.001*shiftY*a3canvas.getNavigationSpeed();
@@ -103,8 +107,6 @@ class WalkController extends A3Controller implements Runnable {
         Quat4d q = new Quat4d(0.0,Math.sin(t),0.0,Math.cos(t));
         cameraQ.mul(q);
         a3canvas.setCameraLocImmediately(cameraV);
-cameraQ.z=0.0;
-cameraQ.normalize();
         a3canvas.setCameraQuatImmediately(cameraQ);
     }
     //NAVIモード(首ふり)
@@ -118,8 +120,18 @@ cameraQ.normalize();
         t = shiftX * -0.001;
         q = new Quat4d(0.0,Math.sin(t),0.0,Math.cos(t));
         cameraQ.mul(q);
-cameraQ.z=0.0;
-cameraQ.normalize();
+        a3canvas.setCameraQuatImmediately(cameraQ);
+    }
+    void rotNaviZ() {
+        Quat4d cameraQ = a3canvas.getCameraQuat();
+        int shiftX = lastMouseX - startMouseX;
+        int shiftY = lastMouseY - startMouseY;
+        double t = shiftY * -0.001;
+        Quat4d q = new Quat4d(Math.sin(t),0.0,0.0,Math.cos(t));
+        cameraQ.mul(q);
+        t = shiftX * -0.001;
+        q = new Quat4d(0.0,Math.sin(t),0.0,Math.cos(t));
+        cameraQ.mul(q);
         a3canvas.setCameraQuatImmediately(cameraQ);
     }
     //NAVIモード(平行移動)
