@@ -56,6 +56,9 @@ class ParserBVH extends DepthFirstAdapter {
         return bone.children.clone();
     }
 
+    public String[] getAllBones() {
+        return bones.keySet().toArray(new String[0]);
+    }
     public Transform3D getTransform3D(String bone,double time) {
         Bone b = bones.get(bone);
         int frame = (int)(time/frameTime);
